package com.xkrs.utils;

import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;


/**
 * @author wudong
 * @date 2021/12/22 17:47
 * @description
 */
public class XlsxUtil {
    private static final Logger log = LoggerFactory.getLogger(XlsxUtil.class);

    /**
     * 获取表格内容, 第一行为表头，第二行为数据，最少两行
     *
     * @param is
     * @return
     */
    public static List<Map<Object, Object>> getXlsxData(InputStream is) {
        List<Map<Object, Object>> list = new ArrayList<>();
        Workbook wb = null;
        try {
            wb = new XSSFWorkbook(is);
        } catch (IOException e) {
            log.error("xlsx，读取数据失败");
            return list;
        }
        Sheet sheet = wb.getSheetAt(0);
        int physicalNumberOfRows = sheet.getPhysicalNumberOfRows();
        if (physicalNumberOfRows < 2) {
            return list;
        }
        Row row = sheet.getRow(0);
        int titNums = row.getPhysicalNumberOfCells();
        Object[] tit = new String[titNums];
        for (int j = 0; j < titNums; j++) {
            Cell cell = row.getCell(j);
            Object value = getValue(cell);
            tit[j] = value;
        }
        for (int i = 1; i < physicalNumberOfRows; i++) {
            Row contentRow = sheet.getRow(i);
            int physicalNumberOfCells = contentRow.getPhysicalNumberOfCells();
            Map<Object, Object> map = new HashMap<>(physicalNumberOfCells);
            for (int j = 0; j < physicalNumberOfCells; j++) {
                Cell cell = contentRow.getCell(j);
                Object value = getValue(cell);
                map.put(tit[j], value);
            }
            list.add(map);
        }
        return list;
    }

    /**
     * 获取值
     *
     * @param cell
     * @return
     */
    private static Object getValue(Cell cell) {
        CellType cellType = cell.getCellType();
        Object value;
        switch (cellType) {
            case STRING:
                value = cell.getStringCellValue().trim();
                break;
            case NUMERIC:
                // 获取时间数据
                if (DateUtil.isCellDateFormatted(cell)) {
                    Date tempValue = cell.getDateCellValue();
                    SimpleDateFormat simpleFormat = new SimpleDateFormat("yyyy-MM-dd");
                    value = simpleFormat.format(tempValue);
                } else {
                    // 防止出现科学计数法
                    DecimalFormat df = new DecimalFormat("0");
                    value = df.format(cell.getNumericCellValue());
                }
                break;
            case BOOLEAN:
                value = cell.getBooleanCellValue();
                break;
            default:
                value = null;
        }
        return value;
    }

    /**
     * 按内容生成xlsx， 不关闭输出流
     *
     * @param contextList 参数
     * @param os
     */
    public static void createXml(List<Map<String, String>> contextList, OutputStream os) {
        // 提取表头
        Set<String> strings = contextList.get(0).keySet();
        Workbook wb = new XSSFWorkbook();
        // 创建一页sheet
        Sheet sheet = wb.createSheet();
        // 创建表头
        Row row = sheet.createRow(0);
        int i = 0;
        for (String key : strings) {
            row.createCell(i++).setCellValue(key);
        }
        // 填写内容
        for (int j = 0; j < contextList.size(); j++) {
            Row ct = sheet.createRow(j + 1);
            ct.setHeight((short) 1200);
            Map<String, String> map = contextList.get(j);
            for (int k = 0; k < map.size(); k++) {
                ct.createCell(k).setCellValue(String.valueOf(map.get(row.getCell(k).getStringCellValue())));
            }
        }
        try {
            // 可以直接写入到zipOutputStream嘛，需要转一下，转为输入流，再进行写入，这样的话就需要把流输出，不能在该地方关闭
            wb.write(os);
            wb.close();
            System.out.println("finish");
        } catch (IOException e) {
            log.error("生成Xlsx失败，IO操作失败， 输出到IO失败", e);
        }
    }

    /**
     * 下载文件  单个
     *
     * @param contextList
     * @param os
     */
    public static void downloadXlsx(List<Map<String, String>> contextList, OutputStream os) {
        createXml(contextList, os);
        try {
            os.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
